;**************************************************************************
;*          1MHz Frequency Counter             *
; Time base is 1 second, generated by TMR2 from the 4MHz crystal osc.
; TMR1 is used for the counter, the overflows are counted in reg freq.
; Every second the UART transmits the 20 bits binary
; frequency count in 3 bytes.   
;
; This code is free to use, for more info   http://www.moty22.co.uk      
;**************************************************************************
#include <p16F628A.inc>

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG      _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _XT_OSC  ; 

;==========================================================================
;       Variable Definition
;==========================================================================

MSEC    EQU .32
FREQ    EQU .33


		ORG	0		;Reset vector address

init
     	MOVLW	B'00000111'	;Disable Comparator module's
		MOVWF	CMCON
		movlw   B'111001'      ;11101
		movwf   T2CON        ;TMR2 prescale=4, postscale=1:8, TMR OFF
		movlw   B'110'    ;110
		movwf   T1CON        ;TMR1 prescale 1:1, osc=off
		MOVLW	B'10010000'	;rx enable 
		MOVWF   RCSTA
		BSF	    STATUS,RP0	;Switch to bank 1
		movlw   .250
		movwf   PR2         ;TMR2 period 4ms
		MOVLW	B'00100000'	;tx status 
		MOVWF   TXSTA
		MOVLW	H'0033'	;baud rate
		MOVWF   SPBRG 
;		MOVLW	B'0'	    
;		MOVWF	OPTION_REG   ;pullup on
		movlw   B'11100010'	
		movwf	TRISB		 ;RB 1-7 inputs
		BCF	    STATUS,RP0	;Switch to Bank 0

COUNT
		MOVLW	.125
		MOVWF	MSEC        ;125 COUNTS FOR 1 SEC
		CLRF	FREQ
		CLRF	TMR2
		CLRF	TMR1L
		CLRF	TMR1H
		BCF		PIR1,TMR1IF
		BCF		PIR1,TMR2IF
		BSF		T1CON,TMR1ON		;OPEN GATE
		BSF		T2CON,TMR2ON		;START COUNT
LOOPA		
		BTFSC	PIR1,TMR1IF         ;ADD 1 TO FREG REG
		GOTO    ADVT1
		BTFSS	PIR1,TMR2IF
		GOTO	LOOPA
		BCF		PIR1,TMR2IF
		DECFSZ	MSEC,F
		GOTO	LOOPA
		BCF		T1CON,TMR1ON		;CLOSE GATE
		BCF		T2CON,TMR2ON		;STOP COUNT
		GOTO	SEND
ADVT1   
		INCF    FREQ,F 				        
		BCF		PIR1,TMR1IF
		GOTO	LOOPA
SEND
		MOVFW   TMR1L
		MOVWF   TXREG          ;TRANSMIT LOW BYTE
		NOP
		NOP
		BTFSS	PIR1,TXIF      ;IS TRANSMIT COMPLETE
		GOTO	$-1
		MOVFW   TMR1H
		MOVWF   TXREG          ;TRANSMIT MID BYTE
		NOP
		NOP
		BTFSS	PIR1,TXIF
		GOTO	$-1		
		MOVFW   FREQ
		MOVWF   TXREG          ;TRANSMIT HIGH BYTE
		NOP
		NOP
		BTFSS	PIR1,TXIF
		GOTO	$-1
		GOTO	COUNT
		
		END
